

local ChunkLoader		= {}
ChunkLoader.__index	= ChunkLoader
setmetatable(ChunkLoader, {__call = function(cls, ...) return cls.new(...) end})
local pairs = pairs



------------------------------------------------------------|
-- Local Functions --
------------------------------------------------------------|
local function updateSpritebatches(layer, chunks)
	if not layer.visible then return end
	local tilesize = layer.map.tilesize

	for i, chunk in pairs(chunks) do
		chunk.spritebatch:clear()
		for y=1, chunk.height/tilesize do
			for x=1, chunk.width/tilesize do
				local layerX = x+(chunk.x*(chunk.width/tilesize)-chunk.width/tilesize)
				local layerY = y+(chunk.y*(chunk.height/tilesize)-chunk.height/tilesize)

				local tid = layer.tiles[layerY][layerX]
				if tid ~= 0 then
					local tile					= layer.map.tileset:getTile(tid)
					local previous_color= {love.graphics.getColor()}
					local color					= layer.color
					local r, g, b				= color[1], color[2], color[3]

					local tx = math.floor(x*tilesize-tilesize)
					local ty = math.floor(y*tilesize-tilesize)

					if tile.quad then
						chunk.spritebatch:setColor(r, g, b, layer.opacity)
						chunk.spritebatch:add(tile.quad, tx, ty)
					elseif tile.animation then
						--> add animation to a table outside of the spritebatch to draw it
					else love.graphics.rectangle("fill", tx, ty, tw, tw) end
					love.graphics.setColor(previous_color)
				end
			end
		end
		chunk.spritebatch:flush()
	end
end



local function newChunk(self, x, y)
	local chunk				= {}
	local tilesize		= self.layer.map.tilesize
	chunk.x						= x
	chunk.y						= y
	chunk.width				= self.chunksettings.tw*tilesize
	chunk.height			= self.chunksettings.th*tilesize
	local maxSprites	= self.chunksettings.tw*self.chunksettings.th
	chunk.spritebatch	= love.graphics.newSpriteBatch(self.layer.map.tileset.image, maxSprites, "dynamic")
	return chunk
end



local function generateChunks(self)
	local chunks = {}
	for y=1, self.chunksettings.ch do
		chunks[y] = {}
		for x=1, self.chunksettings.cw do
			chunks[y][x] = newChunk(self, x, y)
			updateSpritebatches(self.layer, {chunks[y][x]}) -- can be optimized by creating all batches at once
		end
	end
	return chunks
end



------------------------------------------------------------|
-- ChunkLoader Class --
------------------------------------------------------------|
function ChunkLoader.new(layer, chunksettings)
	local self	= setmetatable({}, ChunkLoader)
	self.layer				= layer
	self.chunksettings= chunksettings or {cw=3, ch=3, tw=9, th=9}
	self.chunks				= generateChunks(self)
	return self
end



function ChunkLoader:updateChunks(inputTiles)
	local cw, ch	= self.chunksettings.cw, self.chunksettings.ch
	local tw, th	= self.chunksettings.tw*cw, self.chunksettings.th*ch
	local chunk
	if #inputTiles == 1 then
		local tx, ty = inputTiles[1][1], inputTiles[1][2]
		local cx	= math.max(1, math.ceil(cw/tw*tx))
		local cy	= math.max(1, math.ceil(ch/th*ty))
		print("updating chunk "..cx..", "..cy)
		chunk		= self.chunks[cy][cx]
		updateSpritebatches(self.layer, ({chunk}))
	else
		-- Updating multiple tiles at once --
		local chunks = {}
		for i=1, #inputTiles do
			local tx, ty		= inputTiles[i][1], inputTiles[i][2], inputTiles[i][3]
			local cx				= math.max(1, math.ceil(cw/tw*tx))
			local cy				= math.max(1, math.ceil(ch/th*ty))
			local chunkID		= tostring(cx..cy) -- used to don't have duplicates in the chunks table
			chunks[chunkID]	= self.chunks[cy][cx]
		end
		print("Updating multiple tiles at once")
		updateSpritebatches(self.layer, chunks)
	end
end




function ChunkLoader:draw(camera)
	for y=1, self.chunksettings.ch do
		for x=1, self.chunksettings.cw do

			local chunk = self.chunks[y][x]
			local wx, wy = chunk.width*x-chunk.width, chunk.height*y-chunk.height

			if camera:isVisible(wx, wy, chunk.width, chunk.height) then
				love.graphics.draw(chunk.spritebatch, wx, wy)
				if DEBUG then
					--love.graphics.rectangle("line", wx+64, wy+64, chunk.width-128, chunk.height-128)
					love.graphics.rectangle("line", wx, wy, chunk.width, chunk.height)
					love.graphics.setColor(10, 10, 10, 150)
					love.graphics.rectangle("fill", wx, wy, 64, 16)
					love.graphics.setColor(255, 255, 255, 255)
					love.graphics.print("ID: "..chunk.x..", "..chunk.y, wx, wy)
				end
			end

		end
	end
end

return ChunkLoader
