

local Editor	= {}
local pairs		= pairs
local emptyFunction			= function() end
local screenW, screenH	= love.graphics.getDimensions()
local cam
DEBUG = false

local serialize = require("Editor.ser")



------------------------------------------------------------|
-- Local Functions --
------------------------------------------------------------|
local function getMouseTilePosition(self)
	local cam						= self.camera
	local mx, my				= love.mouse.getPosition()
	return self.map:toTileCoords((mx+cam.x)/cam.scale, (my+cam.y)/cam.scale)
end



function drawTileHighlight(self)
	if self.ui:isOcupied() then return end
	local image		= self.map.tileset:getImage()
	local tile		= self.map.tileset:getTile(self.ui:getSelectedTileID())
	local tw			= self.map.tilesize
	local tx, ty	= getMouseTilePosition(self)
	local wx, wy 	= self.map:toWorldCoords(tx, ty)
	local previousColor = {love.graphics.getColor()}
	love.graphics.setColor(255, 255, 255, 125)
	if tile.quad then love.graphics.draw(image, tile.quad, wx, wy) end
	love.graphics.setColor(255, 255, 255, 25)
	love.graphics.rectangle("line", wx, wy, tw, tw)
	love.graphics.setColor(previousColor)
end



------------------------------------------------------------|
-- Editor --
------------------------------------------------------------|
function Editor:init(data)
	love.filesystem.setIdentity("Editor")
	local Camera	= require("Editor.Camera")
	local Map			= require("Editor.Map")
	local UI			= require("Editor.UI")
	self.map			= Map(data)
	self.ui				= UI(self.map)
	local tw			= self.map.tilesize
	self.camera		= Camera()
	local x, y		= getMouseTilePosition(self)
	self.camera:setPosition(screenW/2-self.map.width/2*tw, screenH/2-self.map.height/2*tw)
	self.isChanging = false
end



function Editor:saveMap(name, data)
	local file = love.filesystem.newFile(name..".ced")
	local data = data or self.map:getSaveData()
	file:open("w")
	file:write(serialize(data))
	file:close()
end



function Editor:loadMap(name)
	if not love.filesystem.exists(name..".ced") then return end
	local data = love.filesystem.load(name..".ced")()
	self:init(data)
end



function Editor:getMap()
	return self.map
end



function Editor:getCamera()
	return self.camera
end



local timer			= 0
local interval	= .0175
local camSpeed	= 700
function Editor:update(dt)
	-- Debug camera movement via arrow keys --
	if love.keyboard.isDown("a") then self.camera:move(-camSpeed*dt, 0) end
	if love.keyboard.isDown("d")then self.camera:move(camSpeed*dt, 0)	end
	if love.keyboard.isDown("w")		then self.camera:move(0, -camSpeed*dt) end
	if love.keyboard.isDown("s") then self.camera:move(0, camSpeed*dt)	end

	-- Update Components --
	self.map:update(dt)
	self.ui:update(dt)

	timer = timer-dt
	if self.edit and timer<=0 and not self.ui:isOcupied() then
		local tx, ty				= getMouseTilePosition(self)
		self.map:setTile(self.map.selectedLayer, tx, ty, self.ui:getSelectedTileID())
		timer = interval
	end
end



function Editor:draw()
	self.camera:set()
	self.map:draw(self.camera)
	drawTileHighlight(self)
	self.camera:unset()
	self.ui:draw()
	-- love.graphics.print("camx, camy: "..self.camera.x..", "..self.camera.y, 10, 30)
end



function Editor:keypressed(key)
	if key == "r" then love.load() end
	if key == "space" then
		if self.map.selectedLayer.id == 1 and #self.map.layers>1 then self.map:setLayer(2)
		else self.map:setLayer(1) end
	end
	if key == "f1" then DEBUG = not DEBUG end
	if key == "f2" then self:saveMap("AwesomeMap") end
	if key == "f3" then self:loadMap("AwesomeMap") end
end



function Editor:mousepressed(x, y, button)
	timer = interval
	if button == 1 then self.edit = true end
	if button == 2 and not self.ui:isOcupied() then
		local tx, ty	= getMouseTilePosition(self)
		local tid			= self.map:getTile(self.map.selectedLayer, tx, ty)
		self.ui:setSelectedTileID(tid)
	end
	self.ui:mousepressed(x, y, button)
end



function Editor:mousereleased(x, y, button)
	self.edit = nil
end



function Editor:wheelmoved(x, y)
	local scaleSpeed = .1
	if self.camera.scale ~= (.1 or 2) then print("Camera Scale: "..self.camera.scale) end
	if y < 0 then self.camera:setScale(math.max(.1, self.camera.scale - scaleSpeed))
	elseif y > 0 then self.camera:setScale(math.min(2, self.camera.scale + scaleSpeed)) end
	local tw 				= self.map.tilesize
	local centerX = screenW/2-self.map.width*self.camera.scale/2*tw
	local centerY = screenH/2-self.map.height*self.camera.scale/2*tw
	self.camera:setPosition(centerX, centerY)
end

return Editor
